/*-----------------------------------------------------------------------

File  : ccl_inferencedoc.c

Author: Stephan Schulz

Contents
 
  Functions and constants for reporting on the proof process.

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Tue Jan  5 16:15:48 MET 1999
    New

-----------------------------------------------------------------------*/

#include "ccl_inferencedoc.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/

#define CLAUSE_FORM true

/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//
// Function: ClauseDocumentCreation()
//
//   If level >= 2, print a minimal creation history and the clause.
//
// Global Variables: -
//
// Side Effects    : Output
//
/----------------------------------------------------------------------*/

void ClauseDocumentCreation(FILE* out, long level, Clause_p clause,
			    char* operation)
{
   if(level>=2)
   {	 
      fprintf(out, "CC %ld %s(", clause->ident, operation);
      if(clause->parent1)
      {
	 fprintf(out, "%ld", clause->parent1->ident);
      }
      if(clause->parent2)
      {
	 putc(',', out);
	 fprintf(out, "%ld", clause->parent2->ident);
      }
      putc(')', out);
      putc(' ', out);
      ClausePrint(out, clause, CLAUSE_FORM);
      putc('\n', out);
   }
}

/*-----------------------------------------------------------------------
//
// Function: ClauseRename()
//
//   Give a new identifier to a clause (and document this in the
//   output if level is >=2. As this function only affects output,
//   it's in ccl_inferencedoc.c and not somewhere else.
//
// Global Variables: -
//
// Side Effects    : Output
//
/----------------------------------------------------------------------*/

void ClauseRename(FILE* out, long level, Clause_p clause, long
		  new_ident)
{
   if(level>=2)
   {
      fprintf(out, "CR %ld RENAME(%ld)\n", new_ident, clause->ident);
   }
   clause->ident = new_ident;
}


/*-----------------------------------------------------------------------
//
// Function: ClauseDocumentModification()
//
//   If level >= 3, print a minimal change history and the clause.
//
// Global Variables: -
//
// Side Effects    : Output
//
/----------------------------------------------------------------------*/

void ClauseDocumentModification(FILE* out, long level, Clause_p clause,
			  char* operation, Clause_p partner)
{
   if(level>=3)
   {	 
      fprintf(out, "CM %ld %s(", clause->ident, operation);
      if(partner)
      {
	 fprintf(out, "%ld", partner->ident);
      }
      putc(')', out);
      putc(' ', out);
      ClausePrint(out, clause, CLAUSE_FORM);
      putc('\n', out);
   }
}


/*-----------------------------------------------------------------------
//
// Function: ClauseDocumentProcessing()
//
//   If level >= 2, print the fact that the clause is being processed
//   in some way, else if level == 1 print the clause as a comment.
//
// Global Variables: -
//
// Side Effects    : Output
//
/----------------------------------------------------------------------*/

void ClauseDocumentProcessing(FILE* out, long level, Clause_p clause,
			      char* operation)
{
   if(level)
   {
      if(level>=2)
      {	 
	 fprintf(out, "CP %ld %s ", clause->ident, operation);
	 ClausePrint(out, clause, CLAUSE_FORM);
	 putc('\n', out);
      }
      else
      {
	 putc('\n', out);
	 putc('#', out);
	 ClausePrint(out, clause, true);
	 putc('\n', out);
      }
   }
}

/*-----------------------------------------------------------------------
//
// Function: ClauseSetDocumentProcessing()
//
//   If level >= 2, print a processing description for all the clauses
//   in the set.
//
// Global Variables: -
//
// Side Effects    : Output
//
/----------------------------------------------------------------------*/

void ClauseSetDocumentProcessing(FILE* out, long level, ClauseSet_p
				 set, char* operation)
{
   Clause_p handle;

   if(level>=2)
   {
      for(handle = set->anchor->succ; handle!=set->anchor; handle =
	     handle->succ)
      {
	 ClauseDocumentProcessing(out, level, handle, operation);
      }
   }   
}

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


