/*-----------------------------------------------------------------------

File  : ccl_inferencedoc.h

Author: Stephan Schulz

Contents
 
  Functions and constants for reporting on the proof process.

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Tue Jan  5 15:27:37 MET 1999
    Partially new, partially lifted from ccl_clauses.[ch]

-----------------------------------------------------------------------*/

#ifndef CCL_INFERENCEDOC

#define CCL_INFERENCEDOC

#include <ccl_clausesets.h>


/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

/* Symbolic constants for the relevant annotations of the output
   format. These should better be valid tokens of type Identifier,
   otherwise some parsing code might break. Except for this
   constraint, change them around as much as you like (but don't
   expect to be compatible with the official release). */ 

#define CC_AXIOM            "AXIOM"
#define CC_COPY             "COPY"
#define CC_PARAMOD          "PM"
#define CC_EFACTOR          "EF"
#define CC_OFACTOR          "OF"
#define CC_ERESOLUTION      "ER"

#define CM_SIMPLIFY_REFLECT "SR"
#define CM_MINIMIZE         "MR"

#define CP_PROCESS          "SELECT"
#define CP_PROOF            "PROOF"
#define CP_FINAL            "FINAL"

#define RW_STANDARD         "RW"



/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

void    ClauseDocumentCreation(FILE* out, long level, Clause_p
			       clause, char* operation);
#define ClauseDefaultDocumentCreation(clause, operation)\
        ClauseDocumentCreation(GlobalOut, OutputLevel, (clause),\
			       (operation))

void    ClauseRename(FILE* out, long level, Clause_p clause, long
		     new_ident);

void    ClauseDocumentModification(FILE* out, long level, Clause_p
				   clause, char* operation, Clause_p
				   partner);
#define ClauseDefaultDocumentModification(clause, operation, partner)\
        ClauseDocumentModification(GlobalOut, OutputLevel, (clause),\
				   (operation), (partner))

void    ClauseDocumentProcessing(FILE* out, long level, Clause_p
				 clause, char* operation);
#define ClauseDefaultDocumentProcessing(clause, operation)\
        ClauseDocumentProcessing(GlobalOut, OutputLevel, (clause),\
				 (operation))

void    ClauseSetDocumentProcessing(FILE* out, long level, ClauseSet_p
				    set, char* operation);

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





