/*-----------------------------------------------------------------------

File  : cco_simplification.c

Author: Stephan Schulz

Contents
 
  Control of simplified clauses

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Mon Jun  8 14:49:49 MET DST 1998
    New

-----------------------------------------------------------------------*/

#include "cco_simplification.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


HCB_p DemodOrderCriterion = NULL;


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//
// Function: DemodInsert()
//
//   Insert a demodulator into the set...
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

void DemodInsert(ClauseSet_p set, Clause_p new)
{
   assert(!new->evaluations);

   if(!set->demod_index)
   {
      assert(DemodOrderCriterion);
      HCBClauseEvaluate(DemodOrderCriterion, new);
      ClauseSetInsert(set, new);
   }
   else
   {
      ClauseSetIndexedInsert(set, new);
   }
}


/*-----------------------------------------------------------------------
//
// Function: DemodExtract()
//
//  Extract a demodulator from the set...
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

Clause_p DemodExtract(Clause_p handle)
{
   assert(handle->set);

   if(!(handle->set->demod_index))
   {
      ClauseSetExtractEntry(handle);
      ClauseRemoveEvaluations(handle);
   }
   else
   {
      ClauseSetIndexedExtractEntry(handle);
   }
   return handle;
}


/*-----------------------------------------------------------------------
//
// Function: RemoveClausesWithRewritableMaxSides()
//
//   Remove all clauses for which potentially maximal terms can be
//   rewritten with new_demod and store them into into. Return number
//   of clauses deleted.
//
// Global Variables: 
//
// Side Effects    : 
//
/----------------------------------------------------------------------*/

long RemoveClausesWithRewritableMaxSides(OCB_p ocb, ClauseSet_p from,
					 ClauseSet_p into, Clause_p
					 new_demod, SysDate nf_date,
					 long *idents)
{
   PTree_p  store = NULL;
   PStack_p stack = PStackAlloc();
   Clause_p handle;
   PTree_p  cell;
   long     res = 0;

   FindClausesWithRewritableMaxSides(ocb, from, &store, new_demod,
				     nf_date);
   
   PStackPushP(stack, store);
   while(!PStackEmpty(stack))
   {
      cell = PStackPopP(stack);
      if(cell)
      {
	 PStackPushP(stack, cell->lson);
	 PStackPushP(stack, cell->rson);
	 handle = cell->key;	 
	 
	 res++;
	 ClauseKillChildren(handle);
	 DemodExtract(handle);
	 ClauseDeleteTermProperties(handle, TPRestricted);	 
	 ClauseRename(GlobalOut, OutputLevel, handle, ++(*idents));
	 ClauseSetInsert(into, handle);
      }
   }
   PStackFree(stack);
   PTreeFree(store);

   return res;
}


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


