/*-----------------------------------------------------------------------

File:    che_lpogen.c

Author:  Joachim Steinbach

Contents:

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes
         New

-----------------------------------------------------------------------*/

#include "che_lpogen.h"


/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/

static void pairgen(Sig_p, Term_p, Term_p, double*);

static void opV(Sig_p, Term_p, double, int, int, double*);

static void myqsort(int, int, double*, FunCode*);

static void swap(int, int, double*, FunCode*);



/*-----------------------------------------------------------------------
//
// Function: myqsort(ops, left, right)
//
//
// Global Variables: -
//
// Side Effects    : sets the global variable finfacG[]
//
-----------------------------------------------------------------------*/

static void myqsort(int left, int right, double* finfacG, FunCode* ops)
{   
   int i, last;

   if (left >= right)
     {
       return;
     }
   
   swap(left, (left+right)/2, finfacG, ops);
   last = left;

   for (i=left+1; i<=right; i++)
     {
       if (finfacG[i] > finfacG[left])
	 {
	   swap(++last, i, finfacG, ops);
	 }
     }

   swap(left, last, finfacG, ops);
   myqsort(left, last-1, finfacG, ops);
   myqsort(last+1, right, finfacG, ops);
}


/*-----------------------------------------------------------------------
//
// Function: swap(left, right)
//
//
// Global Variables: -
//
// Side Effects    : sets the global variable finfacG[]
//
-----------------------------------------------------------------------*/


static void swap(int i, int j, double* finfacG, FunCode* ops) {

   double temp;
   int tmp;

   temp = finfacG[i];
   finfacG[i] = finfacG[j];
   finfacG[j] = temp;

   tmp = ops[i];
   ops[i] = ops[j];
   ops[j] = tmp;
}


/*-----------------------------------------------------------------------
//
// Function: pairgen(sig, left, right)
//
//
// Global Variables: -
//
// Side Effects    : -
//
-----------------------------------------------------------------------*/

static void pairgen(Sig_p sig, Term_p left, Term_p right, double* finfacG)
{

   double *finfacposG, *finfacposcompG;
   double *finfacnegG, *finfacnegcompG;
   double *facG;
   int i, j;

   finfacposG = (double*) SizeMalloc((sig->f_count+1)*sizeof(double));
   finfacposcompG = (double*) SizeMalloc((sig->f_count+1)*sizeof(double));
   finfacnegG = (double*) SizeMalloc((sig->f_count+1)*sizeof(double));
   finfacnegcompG = (double*) SizeMalloc((sig->f_count+1)*sizeof(double));
   facG = (double*) SizeMalloc((sig->f_count+1)*sizeof(double));

   for (i=1; i<=sig->f_count; i++)
   {
      facG[i] = 0.0;           
                 /* initializing the factor array for each operator */
   }
   opV(sig, left, 1, 0, 0, facG);
   for (i=1; i<=sig->f_count; i++)
     {
       if (facG[i] != 0.0)
	 {
	   finfacposG[i] = facG[i];
	   finfacposcompG[i] = 1;
	   for (j=1; j<=sig->f_count; j++)
	     {
	       if ((j != i) && (facG[j] != 0))
		 {
		   finfacposcompG[i] *= facG[j];
		 }
	     }
	 }
       else
	 {
	   finfacposG[i] = finfacposcompG[i] = 0.0;
	 }
     }

   for (i=1; i<=sig->f_count; i++) {
      facG[i] = 0.0;
                 /* initializing the factor array for each operator */
   }
   opV(sig, right, 1, 0, 0, facG);
   for (i=1; i<=sig->f_count; i++)
     {
       if (facG[i] != 0.0)
	 {
	   finfacnegG[i] = facG[i];
	   finfacnegcompG[i] = 0;
	   for (j=1; j<=sig->f_count; j++)
	     {
	       if (j != i)
		 {
		   finfacnegcompG[i] += facG[j];
		 }
	     }
	 }
       else
	 {
	   finfacnegG[i] = finfacnegcompG[i] = 0.0;
	 }
   }

   for (i=1; i<=sig->f_count; i++) {
   finfacG[i] += (finfacposG[i] - finfacnegG[i])*finfacposcompG[i] -
      finfacnegcompG[i];
   }

   SizeFree(finfacposG, (sig->f_count+1)*sizeof(double));
   SizeFree(finfacposcompG, (sig->f_count+1)*sizeof(double));
   SizeFree(finfacnegG, (sig->f_count+1)*sizeof(double));
   SizeFree(finfacnegcompG, (sig->f_count+1)*sizeof(double));
   SizeFree(facG, (sig->f_count+1)*sizeof(double));
}


/*-----------------------------------------------------------------------
//
// Function: opV(t, fac, depth, args)
//
//
// Global Variables: -
//
// Side Effects    : -
//
-----------------------------------------------------------------------*/

static void opV(Sig_p sig, Term_p t, double fac, int depth, int args, double* facG) {

   double newfac;
   int i;

   if (!(TermIsVar(t))) {
      newfac = (depth==0) ? fac++ : fac*pow(2*pow(0.1,depth),args);
      facG[t->f_code] += newfac;
      for (i=0; i<t->arity; i++) {
	 opV(sig, t->args[i], newfac, depth+1, i+1, facG);
      }
   }

}


/*---------------------------------------------------------------------*/
/*                      Exported Functions                             */
/*---------------------------------------------------------------------*/

/*-----------------------------------------------------------------------
//
// Function: LPOGenPrec(sig, list)
//
//
// Global Variables: -
//
// Side Effects    : sets the global variable finfacG[]
//
-----------------------------------------------------------------------*/

void LPOGenPrec(OCB_p ocb, Sig_p sig, ClauseSet_p clset) {

   double *finfacG;
   FunCode *ops;

   int i;
   PStackPointer state;
   Clause_p nextcl;
   Eqn_p eqn;

   ops = (FunCode*) SizeMalloc((sig->f_count)*sizeof(FunCode));
   finfacG = (double*) SizeMalloc((sig->f_count)*sizeof(double));

   for (i=SIG_TRUE_CODE+1; i<sig->f_count; i++) {
      finfacG[i] = 0.0;
                 /* initializing the factor array for each operator */
   }

   for (nextcl=clset->anchor->succ; nextcl!=clset->anchor;
	nextcl=nextcl->succ)
   {
      eqn = nextcl->literals;
      while (eqn)
      {
	 pairgen(sig, eqn->lterm, eqn->rterm, finfacG);
	 eqn = eqn->next;
      }
   }

   for (i=SIG_TRUE_CODE+1; i<=sig->f_count; i++) {
      ops[i] = i;
   }

   myqsort(SIG_TRUE_CODE+1, sig->f_count, finfacG, ops);

   assert(ocb->precedence);
   assert(ocb->sig_size == sig->f_count);

   for(i=SIG_TRUE_CODE+1; i<sig->f_count; i++) 
   {
      state = OCBPrecedenceAddTuple(ocb, ops[i], ops[i+1], to_greater);
      assert(state);
   }
   state = OCBPrecedenceAddTuple(ocb, ops[sig->f_count], SIG_TRUE_CODE, to_greater);
   assert(state);
}


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/
