/*-----------------------------------------------------------------------

File  : cte_termcellstore.c

Author: Stephan Schulz

Contents
 
  Implementation of term cell stores (except for all intersting parts,
  which are realized as macros...

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Mon Oct  5 01:09:50 MEST 1998
    New

-----------------------------------------------------------------------*/

#include "cte_termcellstore.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/

/*-----------------------------------------------------------------------
//
// Function: TermCellStoreInit()
//
//   Initialize a term cell storage.
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

void TermCellStoreInit(TermCellStore store)
{
   int i;
   
   for(i=0; i<TERM_STORE_HASH_SIZE; i++)
   {
      store[i] = NULL;
   }   
}

/*-----------------------------------------------------------------------
//
// Function: TermCellStoreExit()
//
//   Free the trees in a term cell storage.
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

void TermCellStoreExit(TermCellStore store)
{
   int i;
   
   for(i=0; i<TERM_STORE_HASH_SIZE; i++)
   {
      TermTreeFree(store[i]);
      store[i] = NULL;
   }   
}


/*-----------------------------------------------------------------------
//
// Function: TermCellStoreSetProp()
//
//   Set the given properties in all term cells in store.
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

void TermCellStoreSetProp(TermCellStore store, TermProperties props) 
{
   int i;
   
   for(i=0; i<TERM_STORE_HASH_SIZE; i++)
   {
      TermTreeSetProp(store[i], props);
   }   
}

/*-----------------------------------------------------------------------
//
// Function: TermCellStoreDelProp()
//
//   Delete the given properties in all term cells in store.
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

void TermCellStoreDelProp(TermCellStore store, TermProperties props) 
{
   int i;
   
   for(i=0; i<TERM_STORE_HASH_SIZE; i++)
   {
      TermTreeDelProp(store[i], props);
   }   
}


/*-----------------------------------------------------------------------
//
// Function: TermCellStoreNodes()
//
//   Return the number of nodes in the term cell store.
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

long TermCellStoreNodes(TermCellStore store)
{
   long res = 0;
   int i;
   
   for(i=0; i<TERM_STORE_HASH_SIZE; i++)
   {
      res+=TermTreeNodes(store[i]);
   }   
   return res;
}


/*-----------------------------------------------------------------------
//
// Function: TermCellStorePrintDistrib()
//
//   For each entry (hash value) in store, print rhe numbe rof term
//   cells in the corresponding tree.
//
// Global Variables: -
//
// Side Effects    : Output
//
/----------------------------------------------------------------------*/

void TermCellStorePrintDistrib(FILE* out, TermCellStore store)
{
   int i;
   
   for(i=0; i<TERM_STORE_HASH_SIZE; i++)
   {
      printf("# Hash %4d: %6ld\n", i, TermTreeNodes(store[i]));
   }   
}

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


